#include <windows.h>
#include <d3d10.h>
#include <d3dx10.h>
#include "resource.h"

/* create a structure to store the triangles vertex coordinates */
struct TriangleVertex
{
    D3DXVECTOR3 Coordinate;  
};

UINT rectangle_width = 0;
UINT rectangle_height = 0;

/* initialize the instance handle, g_hInstance,
   the handle to the window, g_hWindow, etc. */
HWND                    g_hWindow = NULL;
HINSTANCE               g_hInstance = NULL;  
D3D10_DRIVER_TYPE       g_d3d10driverType = D3D10_DRIVER_TYPE_NULL;
ID3D10Device*           g_id3dDevice = NULL;
IDXGISwapChain*         g_pdx10SwapChain = NULL;
ID3D10RenderTargetView* g_pd3d10RenderTargetView = NULL;

/* function prototypes */
HRESULT InitD3D10();
void DeviceCleanup();
LRESULT CALLBACK WndProc(HWND, UINT, WPARAM, LPARAM);
HRESULT InitWindow(HINSTANCE hInstance, int nCmdShow );
void RenderScene();

/* initialize the message processing loop elements and the idle time required
   for the rendering of our triangle via winMain, the application entry point */
int WINAPI wWinMain(HINSTANCE hInstance, HINSTANCE hPrevInstance, LPWSTR lpCmdLine, int nCmdShow)
{
	if((FAILED(InitWindow(hInstance, nCmdShow)))||(FAILED(InitD3D10())))
	{
		DeviceCleanup(); //do device cleanup  freeing resources
		return 0;
	}

	/* main message loop */
	///////////////////////
	MSG msg = {0};
	
	while(WM_QUIT != msg.message)
	{
		if(PeekMessage(&msg, NULL, 0, 0, PM_REMOVE))
		{
			TranslateMessage(&msg);
			DispatchMessage(&msg);
		}
		else //if no message is returned, do some rendering
			RenderScene();
	}
}

/*
================
InitWindow
-registers the window class and creates the window
================
*/
HRESULT InitWindow(HINSTANCE hInstance, int nCmdShow)
{
	/* register the window class */
	///////////////////////////////
	WNDCLASSEX wndcex;
	wndcex.cbSize 			= sizeof(WNDCLASSEX);
	wndcex.style 			= CS_HREDRAW | CS_VREDRAW;
	wndcex.lpfnWndProc 		= WndProc;
	wndcex.cbClsExtra 		= 0;
	wndcex.cbWndExtra 		= 0;
	wndcex.hInstance 		= hInstance;
	wndcex.hIcon 			= LoadIcon(hInstance, (LPCTSTR)IDI_PROG4_5_2);
	wndcex.hCursor 			= LoadCursor(NULL, IDC_ARROW);
	wndcex.hbrBackground 	= (HBRUSH)(COLOR_WINDOW+1);
	wndcex.lpszMenuName 	= NULL;
	wndcex.lpszClassName 	= "WindowClass";
	wndcex.hIconSm 			= LoadIcon(wndcex.hInstance, (LPCTSTR)IDI_PROG4_5_2);

	if(!RegisterClassEx(&wndcex))
		return E_FAIL;

	/* create the window */
	///////////////////////
	g_hInstance = hInstance;

	RECT rectangle = {0, 0, 400, 300};
	AdjustWindowRect(&rectangle, WS_OVERLAPPEDWINDOW, FALSE);

	g_hWindow = CreateWindow(
					"WindowClass",
					"Initializing Direct3D",
					WS_OVERLAPPEDWINDOW,
					CW_USEDEFAULT, CW_USEDEFAULT,
					rectangle.right - rectangle.left,
					rectangle.bottom - rectangle.top,
					NULL, NULL, hInstance, NULL);

	if(!g_hWindow)
		return E_FAIL;

	ShowWindow(g_hWindow, nCmdShow);

	return S_OK;
}

/*
================
InitD3D10
-create a Direct3D 10 device and swap chain
-HRESULT, the result handle, is 0 for a successful
return from the function and non-zero in the event of an error
================
*/
HRESULT InitD3D10()
{
	//structure for storing the client area coordinates
	RECT rectangle;
	/* get the coordinates of g_hWindows client area and
	write them to the RECT structure */
	GetClientRect(g_hWindow, &rectangle);
	
	/* setup the Direct3D 10 device */
	//////////////////////////////////
	DXGI_SWAP_CHAIN_DESC swapchaindesc;
	
	//fill the memory block with zeros
	SecureZeroMemory(&swapchaindesc, sizeof(swapchaindesc));
	swapchaindesc.BufferCount = 1;
	swapchaindesc.BufferUsage = DXGI_USAGE_RENDER_TARGET_OUTPUT;
	swapchaindesc.BufferDesc.RefreshRate.Numerator = 60;
	swapchaindesc.BufferDesc.RefreshRate.Denominator = 1;
	swapchaindesc.BufferDesc.Width = rectangle_width;
	swapchaindesc.BufferDesc.Height = rectangle_height;
	swapchaindesc.BufferDesc.Format =DXGI_FORMAT_R8G8B8A8_UNORM;
	swapchaindesc.SampleDesc.Count = 1;
	swapchaindesc.SampleDesc.Quality = 0;
	swapchaindesc.OutputWindow = g_hWindow;
	swapchaindesc.Windowed = TRUE;

	// create an interface to the hardware device & swap chain
	HRESULT hresult_ = D3D10CreateDeviceAndSwapChain(
							NULL, g_d3d10driverType, NULL,
							0, D3D10_SDK_VERSION, &swapchaindesc,
							&g_pdx10SwapChain, &g_id3dDevice);
	if(FAILED(hresult_))
		return hresult_;

	/* create the render target */
	//////////////////////////////
	ID3D10Texture2D *pBackBufferSurface;
	hresult_ = g_pdx10SwapChain->GetBuffer(0, __uuidof(ID3D10Texture2D),
										  (LPVOID*)&pBackBufferSurface);

	if(FAILED(hresult_))
		return hresult_;

	hresult_ = g_id3dDevice -> CreateRenderTargetView(pBackBufferSurface, NULL,&g_pd3d10RenderTargetView);

	pBackBufferSurface->Release();

	if(FAILED(hresult_))
		return hresult_;

	g_id3dDevice -> OMSetRenderTargets(1, &g_pd3d10RenderTargetView, NULL);

	/* calculate the viewports width and height from the client
	   area coordinates */
	rectangle_width = rectangle.right - rectangle.left;
	rectangle_height = rectangle.bottom - rectangle.top;

	/* define the viewport */
	/////////////////////////
	D3D10_VIEWPORT viewport;
	viewport.TopLeftX = 0;
	viewport.TopLeftY = 0;
	viewport.Width = rectangle_width;
	viewport.Height = rectangle_height;
	viewport.MinDepth = 0.0f;
	viewport.MaxDepth = 1.0f;
	
	//bind the viewport to the rasterizer stage of the pipeline
	g_id3dDevice -> RSSetViewports(1, &viewport);
	
	return S_OK;
}


/* setup our WndProc callback */
LRESULT CALLBACK WndProc(HWND hWindow, UINT msg, WPARAM wParam, LPARAM lParam)
{
	HDC devicecontext;
	PAINTSTRUCT paintstructure;

	switch(msg)
	{
		case WM_DESTROY:
			PostQuitMessage(0);
			break;

		case WM_PAINT:
			devicecontext = BeginPaint (hWindow, &paintstructure);
			EndPaint(hWindow, &paintstructure);
		break;

	default:
		return DefWindowProc(hWindow, msg, wParam, lParam);
	}
	return 0;
}

/*
================
RenderScene
-clears the back buffer using the colour red
================
*/
void RenderScene()
{
    /* set the clear colour - red, green, blue, alpha*/
    float ClearBufferColour[4] = {1.0f, 0.0f, 0.0f, 1.0f};
   
	/* clear the back buffer */
	g_id3dDevice -> ClearRenderTargetView( g_pd3d10RenderTargetView, ClearBufferColour );
    
	/* switch the back- and front-buffer and display the result */
	g_pdx10SwapChain->Present(0, 0);
}


/*
================
DeviceCleanup
-clean and release all D3D10 objects created
================
*/
void DeviceCleanup()
{
	/* clean and release ID3D10Device */
    if(g_id3dDevice) 
	{
		g_id3dDevice -> ClearState();
		g_id3dDevice -> Release();
	}

	/* release ID3D10RenderTargetView */
    if(g_pd3d10RenderTargetView) 
		g_pd3d10RenderTargetView->Release();
    
	/* release IDXGISwapChain */
	if(g_pdx10SwapChain)
		g_pdx10SwapChain->Release();
}