#include <windows.h>
#include "resource.h"

/*
================
Global Variables
================
*/
HINSTANCE g_hInstance = NULL;
HWND g_hWindow = NULL;

LRESULT CALLBACK WndProc(HWND, UINT, WPARAM, LPARAM);

/*
================
InitWindow
-registers the window class and creates the window
-HRESULT, the result handle, is 0 for a successful return from the function and non-zero in the event of an error
================
*/
HRESULT InitWindow(HINSTANCE hInstance, int nCmdShow)
{
	/* register the window class */
	///////////////////////////////
	WNDCLASSEX wndcex;
	wndcex.cbSize = sizeof(WNDCLASSEX);
	wndcex.style = CS_HREDRAW | CS_VREDRAW;
	wndcex.lpfnWndProc = WndProc;
	wndcex.cbClsExtra = 0;
	wndcex.cbWndExtra = 0;
	wndcex.hInstance = hInstance;
	wndcex.hIcon = LoadIcon(hInstance, (LPCTSTR) IDI_PROG4_5_1);
	wndcex.hCursor = LoadCursor(NULL, IDC_ARROW);
	wndcex.hbrBackground = (HBRUSH)(1);
	wndcex.lpszMenuName = NULL;
	wndcex.lpszClassName = "WindowClass";
	wndcex.hIconSm = LoadIcon(wndcex.hInstance, (LPCTSTR) IDI_PROG4_5_1);

	//if the window class cannot be registered, return an error
	if(!RegisterClassEx(&wndcex))
		return E_FAIL;
	
	/* create the window */
	///////////////////////////////
	
	g_hInstance = hInstance;
	//create a GDI+ rectangle with x = 0, y = 0, width = 400, //and height = 300
	RECT rectangle = {0, 0, 400, 300};
	
	AdjustWindowRect(&rectangle, WS_OVERLAPPEDWINDOW, FALSE);
	
	g_hWindow = CreateWindow(
					"WindowClass",
					"Setting up a Window",
					WS_OVERLAPPEDWINDOW,
					CW_USEDEFAULT,
					CW_USEDEFAULT,
					rectangle.right - rectangle.left,
					rectangle.bottom - rectangle.top,
					NULL, NULL, hInstance, NULL);
	
	//return a standard error if the window creation failed
	if(!g_hWindow)
		return E_FAIL;
	
	//make the window visible
	ShowWindow(g_hWindow, nCmdShow);
	
	//the function has been successful
	return S_OK;
}

/*
================
WndProc
-callback function called whenever an event occurs
and message is received by the application
-LRESULT stores the result of a message processing operation
-CALLBACK is simply a calling convention for callback
functions
-WndProc takes four parameters: a handle to the window
generating the event (hWindow) which is only important when
we have to handle multiple windows with the same window
class, the message id (msg) and two message parameters
(wParam and lParam) defining the message
================
*/
LRESULT CALLBACK WndProc(HWND hWindow, UINT msg, WPARAM wParam, LPARAM lParam)
{
	HDC devicecontext;
	PAINTSTRUCT paintstructure;
	
	switch(msg)
	{
		case WM_PAINT:
			devicecontext = BeginPaint(hWindow, &paintstructure);
			EndPaint(hWindow, &paintstructure);
		break;
	
		case WM_DESTROY:
			PostQuitMessage(0);
		break;
	
		default:
			return DefWindowProc(hWindow, msg, wParam, lParam);
	}
	return 0;
}

int WINAPI wWinMain(HINSTANCE hInstance, HINSTANCE hPrevInstance, LPWSTR lpCmdLine, int nCmdShow)
{
	if(FAILED(InitWindow(hInstance, nCmdShow)))
		return 0;

	/* main message loop */
	///////////////////////
	MSG msg = {0};
	
	while(GetMessage(&msg, NULL, 0, 0))
	{
		TranslateMessage(&msg);
		DispatchMessage(&msg);
	}
	return (int) msg.wParam;
}